/****************************************************************************
*   COPYRIGHT 2017 All rights reserved
*   Panasonic Industrial Devices Europe GmbH
****************************************************************************/

#ifndef __ENVIRONMENT_H__
#define __ENVIRONMENT_H__

/**
* @file configurations/host_mode/application/include/environment.h
* @brief Header file that defines the "environment" for the host mode applications
* @details This header files contains the "environment" for the peripheral and central host mode demo application.
**/

#include "types.h"

/* This is ugly: this file defines struct nvm_partition but is not shared between the host mode and standalone mode
environments. So for the host mode demo we include it here and for the standalone demo it will done accordingly in
the standalone version of environment.h */
#include "application_flash_partitions.h"

/*! Definition of a printf macro that will output directly to the console, without further processing. */
#define xprintf(...)                                                                                                                  \
  do {                                                                                                                                \
    printf(__VA_ARGS__);                                                                                                              \
  } while (0)

#ifdef DEBUG
/*! Definition of the debug printf macro that will output some statistical information as well as the function name
* before the debug message itself */
#define dprintf(...)                                                                                                                  \
  do {                                                                                                                                \
    printf("%s(): ", __func__);                                                                                                       \
    printf(__VA_ARGS__);                                                                                                              \
  } while (0)
#else
/*! Empty definition of the debug printf macro when debugging is not enabled */
#define dprintf(...)
#endif

/**
* @brief Function that will open a UART port that is determined by the following logic: all available FTDI devices will
* be scanned. If one (and only one) UART port is not already opened, then this UART port will be opened.
*
* @param[in] handle Pointer to a handle that will be returned and can be used for further processing like waiting
*
* @return 0 if success, -1 if error
*/
int ftdi_init(HANDLE *handle);

/**
* @brief Function that will change the baudrate of the UART port. The UART port must already be initialized by
* uart_init()
*
* @param[in] baudrate New baudrate to be used
*
* @return 0 if success, -1 if error
*/
int ftdi_set_baudrate(int baudrate);

/**
* @brief Function that enables flow control on the UART. The UART port must already be initialized by
* uart_init()
*
* @return 0 if success, -1 if error
*/
int ftdi_enable_flow_control(void);

#endif