/****************************************************************************
*   COPYRIGHT 2017 All rights reserved
*   Panasonic Industrial Devices Europe GmbH
****************************************************************************/

//#define DEBUG

#include "application.h"
#include "ble_api_slave.h"

/* Use-case specific Beacon ID is from byte 25 to 28 of adv data. RSSI is byte 29 of adv data. */
#define BEACON_ID_OFFSET (25)

/* https://github.com/AltBeacon/spec */
static uint8_t adv_data[] = {
    0x02, 0x01, 0x04, /* ad length, ad type = flags, flags = br/edr not supported */
    0x1b, 0xff,       /* ad length, ad type = custom */
    0x3a, 0x00,       /* mfg id -> Matsushita Electric Industrial Co., Ltd. */
    0xbe, 0xac,       /* beacon code -> 0xBEAC (fixed) */
    'P',  'a',  'n',  'a',  's', 'o', 'n', 'i', 'c', ' ', 'L', 'G', ' ', ' ', ' ', ' ', /* beacon id -> fixed part*/
    0x04, 0xd2, 0x16, 0x2e, /* beacon id -> unique part : 1234 5678 in decimal */
    0xff,                   /* reference rssi */
    0x00,                   /* mfg reserved */
};


void ble_application_slave_get_advertise_data_callback(uint8_t **adv_data_ptr, uint16_t *adv_data_length)
{
  *adv_data_length = sizeof(adv_data);
  *adv_data_ptr = &adv_data[0];
  dprintf("\n");
}

void ble_application_slave_get_scan_response_data_callback(uint8_t **adv_data_ptr, uint16_t *adv_data_length)
{
  *adv_data_length = 0;
  dprintf("\n");
}

void ble_application_slave_set_advertise_data_response_callback(uint8_t status)
{
  enum ble_api_result result;

  dprintf("\n");

  result = ble_api_peripheral_role_start(ADDRESS_CONFIGURATION_PUBLIC, LEGACY_CUSTOM_ADV);
  if (result != LE_API_SUCCESS) {
    warning("ble_api_peripheral_role_start() failed\n");
    return;
  }
}

int beacon_profile_init(uint16_t major, uint16_t minor, uint8_t tx_power)
{
  struct ble_api_legacy_adv_parameters adv_parameter_user;
  enum ble_api_result result;

  memset(&adv_parameter_user, 0x00, sizeof(adv_parameter_user));
  /* Advertising configuration */
  adv_parameter_user.adv_interval_min = 0x00A0;
  adv_parameter_user.adv_interval_max = 0x00A0;
  adv_parameter_user.adv_type = ADV_NONCONN_IND;
  adv_parameter_user.direct_addr_type = SMP_ADDRESS_TYPE_PUBLIC_ADDRESS;
  memset(&adv_parameter_user.direct_bd_addr[0], 0, TCU_SIZE_BDADDR);
  adv_parameter_user.adv_channel_map =
      ADVERTISING_CHANNEL_MAP_CHANNEL_37 | ADVERTISING_CHANNEL_MAP_CHANNEL_38 | ADVERTISING_CHANNEL_MAP_CHANNEL_39;
  adv_parameter_user.adv_filter_policy = LE_SCAN_ANY_CONN_ANY;

  /* Beacon id - from byte 25 to 28 of adv data */
  adv_data[BEACON_ID_OFFSET] = (major >> 8) & 0xff;
  adv_data[BEACON_ID_OFFSET + 1] = (major & 0xff);
  adv_data[BEACON_ID_OFFSET + 2] = (minor >> 8) & 0xff;
  adv_data[BEACON_ID_OFFSET + 3] = (minor & 0xff);

  /* rssi - byte 29 of adv data */
  adv_data[BEACON_ID_OFFSET + 4] = tx_power;

  result = ble_api_configure_legacy_advertise_parameters(&adv_parameter_user);
  if (result) {
    warning("ble_api_configure_legacy_advertise_parameters() failed\n");
    return -1;
  }

  result = ble_api_set_legacy_advertise_data();
  if (result != LE_API_SUCCESS) {
    warning("ble_api_set_legacy_advertise_data() failed\n");
    return -1;
  }

  return 0;
}

int application_init(const uint8_t *device_name, int device_name_len)
{
  int ret;

  ret = beacon_profile_init(1234, 5678, 0xa1);
  if (ret) {
    dprintf("beacon_profile_init() failed\n");
    return -1;
  }

  xprintf("BEACON\n");
  return 0;
}

int application_process(void)
{
  return 0;
}

void ble_application_mng_set_data_length_response_callback(uint8_t status, uint16_t connection_handle)
{
  __debugbreak();
}

/* stubbing - used by central.c, but not peripheral.c */
enum ble_api_result ble_application_master_advertise_report_event_callback(struct tcu_mng_le_adv_report_event *event)
{
  __debugbreak();
  return LE_API_SUCCESS;
}

enum ble_api_result ble_application_master_scan_status_update_callback(enum ble_api_scan_operation operation, uint8_t status)
{
  __debugbreak();
  return LE_API_SUCCESS;
}

enum ble_api_result ble_application_master_scan_disable_response_callback(uint8_t status)
{
  __debugbreak();
  return LE_API_SUCCESS;
}
