/****************************************************************************
*   COPYRIGHT 2017 All rights reserved
*   Panasonic Industrial Devices Europe GmbH
****************************************************************************/

//#define DEBUG

#include "application.h"
#include "ble_api_slave.h"

#include "characteristic_body_sensor_location.h"
#include "characteristic_gap_appearance.h"
#include "characteristic_gap_device_name.h"
#include "characteristic_manufacturer_name_string.h"
#include "characteristic_model_number_string.h"
#include "characteristic_scan_interval_window.h"
#include "characteristic_scan_refresh.h"
#include "characteristic_system_id.h"

#include "server_spp_over_ble_api.h"

#ifdef SPEED_TEST
#define application_process configuration_process
#endif

static const uint8_t manufacturer_name[] = "Panasonic";
static const uint8_t model_number[] = "PAN-USB-ETU";
static const uint8_t manufacturer_identifier[] = {0x01, 0x02, 0x03, 0x04, 0x05};
static const uint8_t organizationally_unique_identifier[] = {0x01, 0x02, 0x03};

int application_init(const uint8_t *device_name, int device_name_len)
{
  int ret;

  server = &spp_over_ble_profile;

  g_characteristic_gap_device_name.length = device_name_len;
  g_characteristic_gap_device_name.name = (uint8_t *)device_name;

  /* device information service */
  g_characteristic_model_number_string.model_number = model_number;
  g_characteristic_model_number_string.length = sizeof(model_number) - 1;

  memcpy(&g_characteristic_system_id.manufacturer_identifier[0], &manufacturer_identifier[0],
         sizeof(g_characteristic_system_id.manufacturer_identifier));
  memcpy(&g_characteristic_system_id.organizationally_unique_identifier[0], &organizationally_unique_identifier[0],
         sizeof(g_characteristic_system_id.organizationally_unique_identifier));

  g_characteristic_manufacturer_name_string.manufacturer_name = (uint8_t *)manufacturer_name;
  g_characteristic_manufacturer_name_string.length = sizeof(manufacturer_name) - 1;

  ret = spp_over_ble_profile_init(device_name, device_name_len);
  if (ret) {
    dprintf("spp_over_ble_profile_init() failed\n");
    return -1;
  }

  dprintf("\n");
  return 0;
}

int configuration_start(void)
{
  enum ble_api_result result;

  result = ble_api_peripheral_role_start(ADDRESS_CONFIGURATION_PUBLIC, LEGACY_AUTO_ADV);
  if (result != LE_API_SUCCESS) {
    dprintf("ble_api_peripheral_role_start() failed\n");
    return -1;
  }

  xprintf("PERIPHERAL\n");
  return 0;
}

#if 0
void ble_application_mng_set_random_address_response_callback(uint8_t status)
{
  int ret;

  ret = configuration_start();
  if (ret) {
    dprintf("configuration_start() failed\n");
    return;
  }

  dprintf("\n");
}
#endif

int application_process(void)
{
  static int once = 0;
  int len = 0;
  int ret;

  if (!once) {
    once = 1;
#if 0
     enum ble_api_result result;
     uint8_t bd_address[] = { 0xcc, 0x13, 0x43, 0xaa, 0xbb, 0x0c };
     result = ble_api_set_random_address(bd_address);
     if (result) {
       dprintf("ble_api_set_random_address() failed\n");
       return -1;
     }

     return 0;
#endif

    ret = configuration_start();
    if (ret) {
      dprintf("configuration_start() failed\n");
      return -1;
    }

    return 0;
  }

  if (!application.is_spp_connected) {
    //    dprintf("not connected - doing nothing\n");
    return 0;
  }

  if (application.is_sending == true) {
    //    dprintf("currently sending - doing nothing\n");
    return 0;
  }

  while (len < application.mtu_size) {
    ret = remove_from_rb(&application.in, &application.buf[len]);
    if (ret) {
      break;
    }
    len++;
  }

  if (!len) {
    //  dprintf("no data to send - doing nothing\n");
    return 0;
  }

  ret = spp_over_ble_send(application.buf, len, application.connection.connection_handle);
  if (ret) {
    dprintf("spp_over_ble_send() failed\n");
    return -1;
  }

  application.is_sending = true;

  dprintf("len %d\n", len);
  return 0;
}

#ifdef SPEED_TEST
int configuration_stop(void)
{
  enum ble_api_result result;

  result = ble_api_peripheral_role_stop();
  if (result != LE_API_SUCCESS) {
    dprintf("ble_api_peripheral_role_stop() failed\n");
    return -1;
  }

  xprintf("STOP\n");
  return 0;
}

int configuration_speed_test_tx(uint8_t *data, uint16_t length, uint16_t connection_handle)
{
  int ret;

  ret = spp_over_ble_send(data, length, connection_handle);
  if (ret) {
    dprintf("spp_over_ble_send() failed\n");
    return -1;
  }

  application.is_sending = true;

  //	dprintf("length %d\n", length);
  return 0;
}

#endif

void ble_application_mng_set_data_length_response_callback(uint8_t status, uint16_t connection_handle)
{
  /* fixme: check status */
  dprintf("status %d\n", status);
}

void spp_over_ble_connect_callback(uint8_t *bd_addr, uint16_t connection_handle, uint16_t mtu_size)
{
  application.mtu_size = mtu_size;
  application.is_spp_connected = true;

  dprintf("mtu_size %d, connection_handle %04x\n", mtu_size, connection_handle);
  xprintf("SPP %d\n", application.mtu_size);
}

void spp_over_ble_mtu_changed_callback(uint8_t *bd_addr, uint16_t connection_handle, uint16_t max_packet_size)
{
  application.mtu_size = max_packet_size;

  dprintf("mtu_size %d, connection_handle %04x\n", mtu_size, connection_handle);
  xprintf("MTU %d\n", application.mtu_size);
}

void spp_over_ble_disconnect_callback(uint16_t connection_handle)
{
  enum ble_api_result result;

  dprintf("connection_handle %04x\n", connection_handle);

  application.is_spp_connected = false;
  application.is_sending = false;

  xprintf("\nDISCONNECT\n");

  result = ble_api_peripheral_role_start(ADDRESS_CONFIGURATION_PUBLIC, LEGACY_AUTO_ADV);
  if (result != LE_API_SUCCESS) {
    dprintf("ble_api_peripheral_role_start() failed\n");
    return;
  }
}

void spp_over_ble_receive_callback(uint8_t *buf, uint16_t length, uint16_t connection_handle)
{
#ifdef SPEED_TEST
  if (application.rx.is_enabled == true) {
    application.rx.total_bytes += length;
    return;
  }
#endif
  xprintf("%.*s", length, buf);
}

void spp_over_ble_sent_callback(enum spp_over_ble_status status, uint16_t connection_handle)
{
#ifdef SPEED_TEST
  if (application.tx.is_enabled == false) {
    dprintf("status %d, connection_handle %04x\n", status, connection_handle);
  }
#endif
  application.is_sending = false;

  /* there may be data left in the buffer, so sent it out now by faking this
   * callback */
  application_process();
}

int server_spp_over_ble_send_data(uint8_t *buf, uint16_t length)
{
  int ret;

  if (!application.is_spp_connected) {
    dprintf("not connected - cannot send data\n");
    return -1;
  }

  if (application.is_sending) {
    dprintf("already sending - cannot send data\n");
    return -1;
  }

  ret = spp_over_ble_send(buf, length, application.connection.connection_handle);
  if (ret) {
    dprintf("spp_over_ble_send() failed\n");
    return -1;
  }

  application.is_sending = true;

  dprintf("length %d\n", length);

  return 0;
}

/* stubbing - used by central.c, but not peripheral.c */
enum ble_api_result ble_application_master_advertise_report_event_callback(struct tcu_mng_le_adv_report_event *event)
{
  __debugbreak();
  return LE_API_SUCCESS;
}

#include "ble_api_master.h"

enum ble_api_result ble_application_master_scan_status_update_callback(enum ble_api_scan_operation operation, uint8_t status)
{
  __debugbreak();
  return LE_API_SUCCESS;
}

enum ble_api_result ble_application_master_scan_disable_response_callback(uint8_t status)
{
  __debugbreak();
  return LE_API_SUCCESS;
}


void ble_application_slave_set_advertise_data_response_callback(uint8_t status)
{
  dprintf("\n");
  __debugbreak();
}

void ble_application_slave_get_advertise_data_callback(uint8_t **adv_data_ptr, uint16_t *adv_data_length)
{
  dprintf("\n");
  __debugbreak();
}

void ble_application_slave_get_scan_response_data_callback(uint8_t **adv_data_ptr, uint16_t *adv_data_length)
{
  dprintf("\n");
  __debugbreak();
}