/****************************************************************************
*   COPYRIGHT 2017 All rights reserved
*   Panasonic Industrial Devices Europe GmbH
****************************************************************************/

#include "application.h"
#include "ble_api.h"

void ble_application_mng_read_whitelist_size_response_callback(uint8_t status, uint8_t size)
{
  dprintf("\n");
  __debugbreak();
}

void ble_application_mng_update_whitelist_response_callback(uint8_t opcode, uint8_t status)
{
  dprintf("\n");
  __debugbreak();
}

void ble_application_mng_generate_oob_data_success_callback(const uint8_t *own_rand_value, const uint8_t *own_confirm_value)
{
  dprintf("\n");
  /* ingore this callback */
  //	__debugbreak();
}

void ble_application_mng_generate_oob_data_failed_callback(uint8_t reason)
{
  dprintf("\n");
  __debugbreak();
}

void ble_application_mng_connect_callback(const struct ble_api_connection *c)
{
  dprintf("bd_address " BD_ADDR_FORMAT_STRING ", conn_interval %d, conn_latency %d, supervision_timeout %d\n",
          BD_ADDR_ARGUMENTS(c->connect_addr), c->conn_interval, c->conn_latency, c->supervision_timeout);
  application.connection = *c;
  application.is_connected = true;
  xprintf("CONNECT " BD_ADDR_FORMAT_STRING "\n", BD_ADDR_ARGUMENTS_BE(application.connection.bd_addr));
}

void ble_application_mng_disconnect_callback(const struct ble_api_connection *connection, uint8_t reason)
{
#ifdef SPEED_TEST
  application.tx.total_bytes = 0;
  application.rx.total_bytes = 0;
#endif
  application.is_connected = false;
  //  application.is_encrypted = false;
  //  application.encryption_setup_failed = false;
  dprintf("reason %02x\n", reason);
}

void ble_application_mng_connection_failed_callback(uint8_t status)
{
  dprintf("status %02x\n", status);
}

int ble_application_mng_update_connection_request_callback(uint16_t connection_handle, struct connection_update_req_event_info *update_parameters)
{
  struct connection_update_accept_param_info update_request_params;
    
  update_request_params.conn_interval_min = update_parameters->conn_interval_min;
  update_request_params.conn_interval_max = update_parameters->conn_interval_max;
  update_request_params.conn_latency = update_parameters->conn_latency;
  update_request_params.sup_time_out = update_parameters->sup_time_out;

  /* automatically accept any incoming update connection request */
  ble_api_remote_connection_parameter_accept_req(connection_handle, LE_UPDATE_ACCEPTED, 0, &update_request_params);
  dprintf("connection_handle %04x\n", connection_handle);
  return 0;
}

int ble_application_mng_remote_connection_parameter_request_callback(uint16_t connection_handle, struct connection_update_req_event_info *update_parameters)
{
  struct connection_update_accept_param_info update_request_params;

  update_request_params.conn_interval_min = update_parameters->conn_interval_min;
  update_request_params.conn_interval_max = update_parameters->conn_interval_max;
  update_request_params.conn_latency = update_parameters->conn_latency;
  update_request_params.sup_time_out = update_parameters->sup_time_out;

  /* automatically accept any incoming remote update connection request */
  ble_api_remote_connection_parameter_accept_req(connection_handle, LE_UPDATE_ACCEPTED, 0, &update_request_params);
  dprintf("connection_handle %04x\n", connection_handle);
  return 0;
}

void ble_application_mng_l2cap_connection_update_callback(uint16_t status, uint16_t connection_handle)
{
  dprintf("\n");
  __debugbreak();
}

void ble_application_mng_connection_update_event_callback(uint8_t *data)
{
  dprintf("\n");
  /* ingore this callback */
  //	__debugbreak();
}

void ble_application_mng_authenticated_payload_timeout_expired_event_callback(uint16_t connection_handle)
{
  dprintf("\n");
  /* ingore this callback */
  //	__debugbreak();
}

void ble_application_mng_get_le_status_response_callback(uint8_t *le_status)
{
  dprintf("\n");
  __debugbreak();
}

void ble_application_mng_read_channel_map_response_callback(uint8_t *channel_map)
{
  dprintf("\n");
  __debugbreak();
}

void ble_application_mng_read_tx_power_level_callback(uint8_t *response)
{
  dprintf("\n");
  __debugbreak();
}

void ble_application_mng_set_host_channel_classification_callback(uint8_t status)
{
  dprintf("\n");
  __debugbreak();
}

void ble_application_mng_read_authenticated_payload_callback(uint8_t *response)
{
  dprintf("\n");
  __debugbreak();
}

void ble_application_mng_write_authenticated_payload_callback(uint8_t *response)
{
  dprintf("\n");
  /* ingore this callback */
  //	__debugbreak();
}

#if 0 
/* these functions are handled differently for peripheral and central because they are used to control the program flow */
void ble_application_mng_set_data_length_response_callback(uint8_t status, uint16_t connection_handle)
{
	__debugbreak();
}
#endif

void ble_application_mng_data_length_change_event_callback(uint8_t status, uint16_t connection_handle,
                                                           struct le_data_length_event *event)
{
  dprintf("max_rx_octets %04x, max_rx_time %04x, max_tx_octets %04x, "
          "max_tx_time %04x\n",
          event->max_rx_octets, event->max_rx_time, event->max_tx_octets, event->max_tx_time);
  //	__debugbreak();
}

void ble_application_mng_read_rssi_callback(uint8_t *response)
{
  dprintf("\n");
  __debugbreak();
}

void ble_api_mng_set_tx_power_response_callback(uint8_t status)
{
  dprintf("\n");
  __debugbreak();
}

void ble_application_mng_gen_non_resolvable_bd_address_failed_callback(uint8_t reason)
{
  dprintf("\n");
  __debugbreak();
}

void ble_application_mng_gen_non_resolvable_bd_address_success_callback(uint8_t *bd_address)
{
  dprintf("\n");
  __debugbreak();
}

void ble_application_mng_set_ir_value_response_callback(uint8_t status)
{
  dprintf("\n");
  __debugbreak();
}

void ble_application_mng_set_random_address_response_callback(uint8_t status)
{
  dprintf("\n");
  __debugbreak();
}


void ble_application_mng_internal_random_address_generated_event_callback(uint8_t status, uint8_t *bd_address)
{
  dprintf("\n");
  __debugbreak();
}

void ble_application_mng_resolve_address_user_failed_callback(void)
{
  dprintf("\n");
  __debugbreak();
}

void ble_application_mng_resolve_address_user_success_callback(uint8_t *data, uint8_t *bd_addr)
{
  dprintf("\n");
  __debugbreak();
}