/****************************************************************************
*   COPYRIGHT 2017 All rights reserved
*   Panasonic Industrial Devices Europe GmbH
****************************************************************************/

#ifndef _USER_CONFIGURATION_H__
#define _USER_CONFIGURATION_H__

/**
* @file user_configuration.h
* @brief Header file that describes the configuration the Bluetooth SDK will be compiled with
* @details This header files describes the configuration the Bluetooth SDK will be compiled with,
* for example, which ROM version should be used and if central and/or peripheral or client and/or
* server handling should be compiled in.
**/

/*! Definition that selects support for ROM version 002 of PAN1760A */
#define ROM_002

/*! Definition that configures support for central role by setting the max. number of remote masters */
#define MAX_REMOTE_MASTER_COUNT 1

/*! Definition that configures support for peripheral role by setting the max. number of remote slaves */
#define MAX_REMOTE_SLAVE_COUNT 1

/*! Definition that configures support for GATT server */
#define GATT_ROLE_SERVER 0x01

/*! Definition that configures support for GATT client */
#define GATT_ROLE_CLIENT 0x01

/*! Definition that enables feature switch */
#define BLE_INIT_COMMAND_FORMAT 0x01

/*! Definition that configures support for feature switch format */
#define FEATURE_SWITCH_FORMAT 0x03

/*! Definition that configures support for LE secure connection */
#define LE_SECURE_CONNECTION_SUPPORT 0x01

/*! Definition that configures support for connection parameter request */
#define LL_CONNECTION_PARAMETER_REQUEST_SUPPORT 0x01

/*! Specify Maximum possible data size of GATT operations (char length, write data size etc)
 Set as Maximum MTU if application want to make use of Complete MTU supported by the BT Chip. */
#define MAX_GATT_DATA_SIZE 247

#endif
