/****************************************************************************
*   COPYRIGHT 2017 All rights reserved
*   Panasonic Industrial Devices Europe GmbH
****************************************************************************/

#ifndef __ENVIRONMENT_H__
#define __ENVIRONMENT_H__

/**
* @file configurations/standalone_mode/application/include/environment.h
* @brief Header file that defines the "environment" for the standalone mode applications
* @details This header files contains the "environment" for the peripheral and central standalone mode demo application.
**/

#include "types.h"
/* We don't want to modify types.h because it is from the SDK, so additional includes go here */
#include <stdbool.h>

/*! Definition of debugger break macro that comes from Windows. Replace it with a Cortex-M0 intrinsic */
#define __debugbreak(...) __asm("bkpt #0\n");

/**
* @brief Function that will output the supplied format and arguments unconditionally to UART1.
*
* @param[in] format printf()-like format string
*
*/
void uart1_printf(const char *format, ...);

/*! Definition of a printf macro that will output directly to UART1, without further processing. */
#define xprintf(...)                                                                                                                  \
  do {                                                                                                                                \
    uart1_printf(__VA_ARGS__);                                                                                                        \
  } while (0)

#ifndef DEBUG
/*! Empty definition of the debug printf macro when debugging is not enabled */
#define dprintf(...)
#else
/*! Definition of the debug printf macro that will output some statistical information as well as the function name
* before the debug message itself */
#define dprintf(...)                                                                                                                  \
  do {                                                                                                                                \
    uart1_printf("%s(): ", __func__);                                                                                                 \
    uart1_printf(__VA_ARGS__);                                                                                                        \
  } while (0)

#endif

#ifndef NDEBUG
/*! If NDEBUG is not defined, the warning() function will output information */
#define warning(...)                                                                                                                  \
  do {                                                                                                                                \
    uart1_printf("[W] %s(): ", __func__);                                                                                             \
    uart1_printf(__VA_ARGS__);                                                                                                        \
  } while (0)
#else
/*! If NDEBUG is defined, not even the warning() function will output any information */
#define warning(...)
#endif

#endif